{************************************************************************}
{ spgpKeyGen - key-generation functions                                  }
{                                                                        }
{ This unit requires one or more units from the Pascal translations of   }
{ the PGPsdk header files: <http://www.oz.net/~srheller/pgpsdk/>         }
{ 								                                                       }
{ The original Pascal code is: spgpKeyGen.pas, released 10 Apr 2000.     }
{ The initial developer of the Pascal code is Steven R. Heller           }
{ <srheller@oz.net>                                                      }
{ 								                                                       }
{ Portions created by Steven R. Heller are    				                   }
{ Copyright (C) 2000 Steven R. Heller                                    }
{									                                                       }
{ The contents of this file are used with permission, subject to         }
{ the Mozilla Public License Version 1.1 (the "License"); you may        }
{ not use this file except in compliance with the License. You may       }
{ obtain a copy of the License at                                        }
{ http://www.mozilla.org/MPL/MPL-1.1.html 	                             }
{ 									                                                     }
{ Software distributed under the License is distributed on an 	         }
{ "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or         }
{ implied. See the License for the specific language governing           }
{ rights and limitations under the License.                              }
{                                                                        }
{************************************************************************}

unit spgpKeyGen;

interface

uses
  // Delphi
  SysUtils,
  // PGPsdk
  pgpBase, pgpPubTypes, pgpOptionList, pgpUserInterface,
    pgpKeys, pgpFeatures, pgpSDKPrefs, pgpUtilities,
    pgpRandomPool, pgpEncodePas, pgpErrors, pgpClient,
  // SPGP
  spgpUtil, spgpGlobals, spgpKeyUtil,
  // always last
  PGPsdk;

  function spgpkeygenerate(UserID, PassPhrase, NewKeyHexID: PChar;
           KeyAlg, CipherAlg, Size, ExpiresIn, FastGeneration, FailWithoutEntropy,
           WinHandle: Longint): LongInt; stdcall; export;
  function spgpsubkeygenerate(MasterKeyHexID, MasterKeyPass, NewSubKeyHexID: PChar;
           KeyAlg, Size: Longint; ExpiresIn, FastGeneration, FailWithoutEntropy,
           WinHandle: Longint): LongInt; stdcall; export;

implementation

var
  LibsAllLoaded  : boolean;
  PGPversion5    : boolean;

function KeyGenEventHandler(Context: pPGPContext; Event: pPGPEvent; UserValue: PGPUserValue): PGPError; cdecl;
begin
  result := 0;
  case event^.etype of
    kPGPEvent_KeyGenEvent            :;
    kPGPEvent_EntropyEvent           :;
  end;
end;

function MyKeyGenEventHandler(Context: pPGPContext; Event: pPGPEvent;
  UserValue: PGPUserValue):PGPError; cdecl;
begin
  result := KeyGenEventHandler(Context, Event, UserValue);
end;

(* GENERATE KEY PAIR *)
function spgpkeygenerate(
         UserID         : PChar;   // name + <e-mail>
         PassPhrase     : PChar;
         NewKeyHexID    : PChar;   // hex id of the generated key
         KeyAlg         : Longint; // pub. key algorithm
         CipherAlg      : Longint; // preferred ciphering algorithm
         Size           : Longint; // size in bits
         ExpiresIn      : Longint; // expiration in days-from-now
         FastGeneration : Longint;
         FailWithoutEntropy : Longint; // fail with error, or gather entropy via dialog?
         WinHandle      : Longint
         ): LongInt; stdcall; export;
var
  AlgCount       : PGPUInt32;
  Context        : pPGPContext;
  EntropyNeeded  : PGPUInt32;
  KeySet         : pPGPKeySet;
  NewKey         : pPGPKey;
  PreferredAlg   : PGPCipherAlgorithm;
  PubKeyAlg      : PGPPublicKeyAlgorithm;
  s              : shortstring;
  v              : PGPUint32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;

  result := Init(Context, KeySet, true, true);

  if result <> 0 then exit;

  try
    // convert parameters to sdk type
    case KeyAlg of
      1: PubKeyAlg := kPGPPublicKeyAlgorithm_RSA;
      2: PubKeyAlg := kPGPPublicKeyAlgorithm_RSAEncryptOnly;
      3: PubKeyAlg := kPGPPublicKeyAlgorithm_RSASignOnly;
      4: PubKeyAlg := kPGPPublicKeyAlgorithm_ElGamal;// A.K.A.Diffie-Hellman
      5: PubKeyAlg := kPGPPublicKeyAlgorithm_DSA
    else
      PubKeyAlg := kPGPPublicKeyAlgorithm_Invalid;
    end;

    case CipherAlg of
      0:
      case PubKeyAlg of
        kPGPPublicKeyAlgorithm_RSA: PreferredAlg := kPGPCipherAlgorithm_IDEA;
        kPGPPublicKeyAlgorithm_DSA: PreferredAlg := kPGPCipherAlgorithm_CAST5
      else
        PreferredAlg := kPGPCipherAlgorithm_IDEA;
      end;
      1: PreferredAlg := kPGPCipherAlgorithm_IDEA;
      2: PreferredAlg := kPGPCipherAlgorithm_3DES;
      3: PreferredAlg := kPGPCipherAlgorithm_CAST5
    else
      PreferredAlg := kPGPCipherAlgorithm_IDEA;
    end;

    // Entropy: 1. calculate entropy needed
    EntropyNeeded := PGPGetKeyEntropyNeeded(Context,
      [
      PGPOKeyGenParams(Context, PubKeyAlg, Size),
      PGPOKeyGenFast(Context, PGPBoolean(FastGeneration))
      ]);

    // 2. is it enough, and do we collect or bail out?
    if (EntropyNeeded >= PGPGlobalRandomPoolGetEntropy)
    and not Boolean(FailWithoutEntropy) then
    begin
      if PGPversion5 and Assigned(PGPcomdlgRandom) then
        result := PGPcomdlgRandom(Context, EntropyNeeded, WinHandle)
      else
        result := PGPCollectRandomDataDialog(Context, EntropyNeeded,
          [PGPOUIParentWindowHandle(Context, WinHandle)]);
    end;

    if result <> 0 then exit;

    result := PGPCountSymmetricCiphers(AlgCount);

    if result <> 0 then exit;
    
    result := PGPGenerateKey(Context,
      NewKey,
      [
      PGPOKeyGenParams(Context, PubKeyAlg, Size),
      PGPOKeyGenName(Context, UserID, Length(UserID)),
      PGPOKeySetRef(Context, KeySet),
      PGPOPreferredAlgorithms(Context, PreferredAlg, AlgCount),
      PGPOPassphrase(Context, Passphrase),
      PGPOKeyGenFast(Context, PGPBoolean(FastGeneration)),
      PGPOExpiration(Context, ExpiresIn),
      PGPOEventHandler(Context, KeyGenEventHandler, nil)
      ]);

    if result <> 0 then exit;

    // save to file
    if Boolean(PGPKeySetNeedsCommit(KeySet)) then
      result := PGPCommitKeyRingChanges(KeySet);

    if result <> 0 then exit;

    StrPCopy(NewKeyHexID, '');

    // extract the new key's hex id
    result := GetKeyPropKeyID(NewKey, s);

    if result <> 0 then exit;

    // copy hex id to output buffer
    StrPLCopy(NewKeyHexID, s, 255);

  finally
    if Assigned(PGPcomdlgCloseLibrary) then
      PGPcomdlgCloseLibrary;
    QuitIt(Context, KeySet);
  end;
end;

(* GENERATE SUB-KEY *)
function spgpsubkeygenerate(
         MasterKeyHexID    : PChar;
         MasterKeyPass     : PChar;
         NewSubKeyHexID    : PChar;   // hex id of the generated key
         KeyAlg            : Longint; //pub.key algorithm
         Size              : Longint; //bits of key/ciphering key
         ExpiresIn         : Longint; //expiration in days from now
         FastGeneration    : Longint;
         FailWithoutEntropy : Longint;
         WinHandle         : Longint
         ): LongInt;stdcall;export;
var
  Context        : pPGPContext;
  EntropyNeeded  : PGPUInt32;
  KeySet         : pPGPKeySet;
  MasterKey      : pPGPKey;
  NewSubKey      : pPGPSubKey;
  PubKeyAlg      : PGPPublicKeyAlgorithm;
  s              : shortstring;
  v              : PGPUint32;
begin
  result := kPGPError_NoError;

  if not LibsAllLoaded then
    result := SPGPError_NoPGPsdk;

  if result <> 0 then exit;

  Context := nil;
  KeySet := nil;
  MasterKey := nil;

  result := Init(Context, KeySet, true, true);

  if result <> 0 then exit;

  try
    // get master key from hex id
    result := GetKeyByID(Context, MasterKey, StrPas(MasterKeyHexID), KeySet);

    if result <> 0 then exit;

    case KeyAlg of
      1: PubKeyAlg := kPGPPublicKeyAlgorithm_RSA;           //1
      2: PubKeyAlg := kPGPPublicKeyAlgorithm_RSAEncryptOnly;//2
      3: PubKeyAlg := kPGPPublicKeyAlgorithm_RSASignOnly;   //3
      4: PubKeyAlg := kPGPPublicKeyAlgorithm_ElGamal;       //16// A.K.A.Diffie-Hellman
      5: PubKeyAlg := kPGPPublicKeyAlgorithm_DSA            //17
    else
      PubKeyAlg := kPGPPublicKeyAlgorithm_Invalid;
    end;

    // 1. calculate needed entropy
    EntropyNeeded := PGPGetKeyEntropyNeeded(Context,
      [
      PGPOKeyGenParams(Context, PubKeyAlg, Size),
      PGPOKeyGenFast(Context, PGPBoolean(FastGeneration))
      ]);

    // 2. is it enough, and do we collect or bail out?
    if (EntropyNeeded >= PGPGlobalRandomPoolGetEntropy)
    and not Boolean(FailWithoutEntropy) then
    begin
      if PGPversion5 and Assigned(PGPcomdlgRandom) then
        //result := PGP5Dialog(EntropyNeeded, WinHandle)
        result := PGPcomdlgRandom(Context, EntropyNeeded, WinHandle)
      else
        result := PGPCollectRandomDataDialog(Context, EntropyNeeded,
          [PGPOUIParentWindowHandle(Context, WinHandle)]);
    end;

    if result <> 0 then exit;

    result := PGPGenerateSubKey(Context,
      NewSubKey,
      [
      PGPOKeyGenMasterKey(Context, MasterKey),
      PGPOKeyGenParams(Context, PubKeyAlg, Size),
      PGPOPassphrase(Context, MasterKeyPass),
      PGPOKeyGenFast(Context, PGPBoolean(FastGeneration)),
      PGPOExpiration(Context, ExpiresIn)
      ]);

    if result <> 0 then exit;

    // save to file
    if Boolean(PGPKeySetNeedsCommit(KeySet)) then
      result := PGPCommitKeyRingChanges(KeySet);

    if result <> 0 then exit;

    StrPCopy(NewSubKeyHexID, '');

    // extract the new key's hex id
    result := GetSubKeyPropKeyID(NewSubKey, s);

    if result <> 0 then exit;

    // copy hex id to output buffer
    StrPLCopy(NewSubKeyHexID, s, 255);

  finally
    if Assigned(PGPcomdlgCloseLibrary) then
      PGPcomdlgCloseLibrary;
    QuitIt(Context, KeySet);
  end;
end;

initialization
  // check all pgp units used by this unit
  LibsAllLoaded := pgpUtilitiesLoaded and
                   pgpSDKPrefsLoaded and
                   pgpErrorsLoaded and
                   pgpKeysLoaded and
                   pgpOptionListLoaded and
                   //pgpUserInterfaceLoaded and
                   pgpFeaturesLoaded and
                   pgpRandomPoolLoaded and
                   pgpEncodeLoaded;// and
                   //pgpClientLoaded;

  PGPversion5 := IsPGP5;
end.
